@@ -872,12 +872,12 @@ nc_write_error(struct wclb_arg *arg, struct nc_server_error *err)
         nc_write_clb((void *)arg, "</error-message>", 16, 0);
     }
 
-    if (err->sid || err->attr_count || err->elem_count || err->ns_count || err->other_count) {
+    if ((err->sid > -1) || err->attr_count || err->elem_count || err->ns_count || err->other_count) {
         nc_write_clb((void *)arg, "<error-info>", 12, 0);
 
-        if (err->sid) {
+        if (err->sid > -1) {
             nc_write_clb((void *)arg, "<session-id>", 12, 0);
-            sprintf(str_sid, "%u", err->sid);
+            sprintf(str_sid, "%u", (uint32_t)err->sid);
             nc_write_clb((void *)arg, str_sid, strlen(str_sid), 0);
             nc_write_clb((void *)arg, "</session-id>", 13, 0);
         }
@@ -976,7 +976,8 @@ nc_write_msg(struct nc_session *session, NC_MSG_TYPE type, ...)
             nc_write_clb((void *)&arg, "<ok/>", 5, 0);
             break;
         case NC_RPL_DATA:
-            lyd_print_clb(nc_write_xmlclb, (void *)&arg, ((struct nc_reply_data *)reply)->data, LYD_XML, LYP_WITHSIBLINGS);
+            assert(((struct nc_reply_data *)reply)->data->schema->nodetype == LYS_RPC);
+            lyd_print_clb(nc_write_xmlclb, (void *)&arg, ((struct nc_reply_data *)reply)->data->child, LYD_XML, LYP_WITHSIBLINGS);
             break;
         case NC_RPL_ERROR:
             error_rpl = (struct nc_server_reply_error *)reply;
