@@ -140,6 +140,7 @@ nc_err(NC_ERR tag, ...)
         ERRMEM;
         return NULL;
     }
+    ret->sid = -1;
 
     va_start(ap, tag);
 
@@ -502,6 +503,28 @@ nc_err_libyang(void)
     return e;
 }
 
+API NC_ERR_TYPE
+nc_err_get_type(struct nc_server_error *err)
+{
+    if (!err) {
+        ERRARG("err");
+        return 0;
+    }
+
+    return err->type;
+}
+
+API NC_ERR
+nc_err_get_tag(struct nc_server_error *err)
+{
+    if (!err) {
+        ERRARG("err");
+        return 0;
+    }
+
+    return err->tag;
+}
+
 API int
 nc_err_set_app_tag(struct nc_server_error *err, const char *error_app_tag)
 {
@@ -521,6 +544,17 @@ nc_err_set_app_tag(struct nc_server_error *err, const char *error_app_tag)
     return 0;
 }
 
+API const char *
+nc_err_get_app_tag(struct nc_server_error *err)
+{
+    if (!err) {
+        ERRARG("err");
+        return NULL;
+    }
+
+    return err->apptag;
+}
+
 API int
 nc_err_set_path(struct nc_server_error *err, const char *error_path)
 {
@@ -540,6 +574,17 @@ nc_err_set_path(struct nc_server_error *err, const char *error_path)
     return 0;
 }
 
+API const char *
+nc_err_get_path(struct nc_server_error *err)
+{
+    if (!err) {
+        ERRARG("err");
+        return 0;
+    }
+
+    return err->path;
+}
+
 API int
 nc_err_set_msg(struct nc_server_error *err, const char *error_message, const char *lang)
 {
@@ -568,15 +613,23 @@ nc_err_set_msg(struct nc_server_error *err, const char *error_message, const cha
     return 0;
 }
 
+API const char *
+nc_err_get_msg(struct nc_server_error *err)
+{
+    if (!err) {
+        ERRARG("err");
+        return 0;
+    }
+
+    return err->message;
+}
+
 API int
 nc_err_set_sid(struct nc_server_error *err, uint32_t session_id)
 {
     if (!err) {
         ERRARG("err");
         return -1;
-    } else if (!session_id) {
-        ERRARG("session_id");
-        return -1;
     }
 
     err->sid = session_id;
